@@ -2,11 +2,13 @@
 # code for this file is in Libpng.xs in the top directory.
 
 package Image::PNG::Libpng;
+use warnings;
+use strict;
+
 require Exporter;
-require DynaLoader;
 use Carp;
-@ISA = qw(Exporter);
-@EXPORT_OK = qw/
+our @ISA = qw(Exporter);
+our @EXPORT_OK = qw/
 	create_read_struct
 	create_write_struct
 	destroy_read_struct
@@ -29,7 +31,6 @@ use Carp;
 	get_row_pointers
 	get_rows
 	get_rowbytes
-	get_channels
 	get_valid
 	set_tRNS_pointer
 	set_rows
@@ -87,14 +88,15 @@ read_png_file
 write_png_file
 color_type_name
 get_internals
+copy_png
 /;
 
-%EXPORT_TAGS = (
+our %EXPORT_TAGS = (
     all => \@EXPORT_OK,
 );
 
 require XSLoader;
-our $VERSION = '0.28';
+our $VERSION = '0.29';
 
 XSLoader::load('Image::PNG::Libpng', $VERSION);
 
@@ -114,8 +116,11 @@ sub write_file
 
 sub read_png_file
 {
-    my ($file_name) = @_;
+    my ($file_name, %options) = @_;
     my $png = create_read_struct ();
+    if ($options{transforms}) {
+	$png->set_transforms ($options{transforms});
+    }
     open my $in, "<:raw", $file_name
         or croak "Cannot open '$file_name' for reading: $!";
     $png->init_io ($in);
@@ -196,9 +201,54 @@ sub get_chunk
 
 sub set_chunk
 {
-
+    my ($png, $chunk, $value) = @_;
+    if ($chunk eq 'IDAT') {
+	croak "Use get_rows";
+    }
+    if ($known_chunks{$chunk}) {
+	no strict 'refs';
+	my $sub = "set_$chunk";
+	return &$sub ($png, $value); 
+    }
+    croak "Unknown chunk $chunk";
 }
 
+sub copy_png
+{
+    my ($png, %options) = @_;
+    my $opng = create_write_struct ();
+    if ($options{verbosity}) {
+	$opng->set_verbosity ($options{verbosity});
+    }
+    my $valid = $png->get_valid ();
+    $opng->set_IHDR ($png->get_IHDR ());
+    for my $chunk (keys %$valid) {
+	if ($chunk eq 'IHDR') {
+	    next;
+	}
+	if ($chunk eq 'IDAT') {
+	    my $rows = get_rows ($png);
+	    $opng->set_rows ($rows);
+	}
+	elsif ($valid->{$chunk}) {
+	    $opng->set_chunk ($chunk, $png->get_chunk ($chunk));
+	}
+    }
+    return $opng;
+}
+
+
+sub width
+{
+    goto & get_image_width;
+}
+
+sub height
+{
+    goto & get_image_height;
+}
+
+
 
 1;
 
